/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*

Generation of biased "random numbers", according to the psychological law:
- if two bits are drawn at randim, then the human mind expects that the second one will be different from the first one
with a probability greater than 0.5 (typically about 0.7).

 * main.c
 * Copyright (C) 2012 Maurice <Maurice.Clerc@WriteMe.com>
2012-09-04
	 * 
 * Biased_RNG is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * Biased_RNG is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "math.h"
#include <stdlib.h>
#define bMax 8

double alea(double a, double b); 
FILE * f_trace;

int main()
{
	int b;
	double bias;
	int bits[bMax];
	int m;
	double r;
	int randNb;
	double randNum[100];
	double zz;
	f_trace = fopen ("f_trace.txt", "w");

	bias=0.7; // 0.5 => no bias 
	randNb=97; // Number of numbers to generate

	if(alea(0,1)<0.5) bits[0]=0; else bits[0]=1; // Very first bit at random

	for(m=0; m<randNb;m++)
	{
		zz=bits[0];
		// Built a sequence of bMax bits 
		// and say it is the binary representation of a real number
		for(b=1;b<bMax;b++)
		{
			r=alea(0,1);
			if(r<bias)
				bits[b]=1-bits[b-1];
			else bits[b]=bits[b-1];

			if(bits[b]==1) zz=zz+pow(2,b);

		}
		randNum[m]=zz;

		// Prepare the next number
		if(alea(0,1)<bias) bits[0]=1-bits[bMax-1];
		else bits[0]=bits[bMax-1];	
	}

	// Scale
	zz=pow(2,bMax)-1;
	for(m=0;m<randNb; m++) randNum[m]=randNum[m]/zz;

	// Display and save
	for(m=0;m<randNb; m++)
	{
		printf("\n %f",randNum[m]);
		fprintf(f_trace,"%f\n",randNum[m]);
	}
	return (0);
}
//==============================================================================
double alea(double a, double b)
{
	double r;
	r=(double)rand()/RAND_MAX;
	r=a+r*(b-a);
	return r;
}
